@@ -1,8 +1,8 @@
 /* dn.c - routines for dealing with distinguished names */
-/* $OpenLDAP: pkg/ldap/servers/slapd/dn.c,v 1.170.2.7 2005/12/21 00:06:30 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/dn.c,v 1.170.2.12 2007/08/04 20:35:38 hyc Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2005 The OpenLDAP Foundation.
+ * Copyright 1998-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -262,81 +262,62 @@ rdnValidate(
  * Note: the sorting can be slightly improved by sorting first
  * by attribute type length, then by alphabetical order.
  *
- * uses a linear search; should be fine since the number of AVAs in
+ * uses an insertion sort; should be fine since the number of AVAs in
  * a RDN should be limited.
  */
-static void
-AVA_Sort( LDAPRDN rdn, int iAVA )
+static int
+AVA_Sort( LDAPRDN rdn, int nAVAs )
 {
+	LDAPAVA	*ava_i;
 	int		i;
-	LDAPAVA		*ava_in = rdn[ iAVA ];
 
 	assert( rdn != NULL );
-	assert( ava_in != NULL );
-	
-	for ( i = 0; i < iAVA; i++ ) {
-		LDAPAVA		*ava = rdn[ i ];
-		int		a, j;
 
-		assert( ava != NULL );
+	for ( i = 1; i < nAVAs; i++ ) {
+		LDAPAVA *ava_j;
+		int j;
 
-		a = strcmp( ava_in->la_attr.bv_val, ava->la_attr.bv_val );
+		ava_i = rdn[ i ];
+		for ( j = i-1; j >=0; j-- ) {
+			int a;
 
-		if ( a > 0 ) {
-			break;
-		}
+			ava_j = rdn[ j ];
+			a = strcmp( ava_i->la_attr.bv_val, ava_j->la_attr.bv_val );
 
-		while ( a == 0 ) {
-			int		v, d;
+			if ( a == 0 ) {
+				int		d;
 
-			d = ava_in->la_value.bv_len - ava->la_value.bv_len;
+				d = ava_i->la_value.bv_len - ava_j->la_value.bv_len;
 
-			v = memcmp( ava_in->la_value.bv_val, 
-					ava->la_value.bv_val,
-					d <= 0 ? ava_in->la_value.bv_len 
-						: ava->la_value.bv_len );
+				a = memcmp( ava_i->la_value.bv_val, 
+						ava_j->la_value.bv_val,
+						d <= 0 ? ava_i->la_value.bv_len 
+							: ava_j->la_value.bv_len );
 
-			if ( v == 0 && d != 0 ) {
-				v = d;
+				if ( a == 0 ) {
+					a = d;
+				}
 			}
+			/* Duplicates are not allowed */
+			if ( a == 0 )
+				return LDAP_INVALID_DN_SYNTAX;
 
-			if ( v <= 0 ) {
-				/* 
-				 * got it!
-				 */
+			if ( a > 0 )
 				break;
-			}
 
-			if ( ++i == iAVA ) {
-				/*
-				 * already sorted
-				 */
-				return;
-			}
-
-			ava = rdn[ i ];
-			a = strcmp( ava_in->la_attr.bv_val, 
-					ava->la_attr.bv_val );
+			rdn[ j+1 ] = rdn[ j ];
 		}
-
-		/*
-		 * move ahead
-		 */
-		for ( j = iAVA; j > i; j-- ) {
-			rdn[ j ] = rdn[ j - 1 ];
-		}
-		rdn[ i ] = ava_in;
-
-		return;
+		rdn[ j+1 ] = ava_i;
 	}
+	return LDAP_SUCCESS;
 }
 
 static int
 LDAPRDN_rewrite( LDAPRDN rdn, unsigned flags, void *ctx )
 {
 
-	int rc;
-	int		iAVA;
+	int rc, iAVA, do_sort = 0;
+
 	for ( iAVA = 0; rdn[ iAVA ]; iAVA++ ) {
 		LDAPAVA			*ava = rdn[ iAVA ];
 		AttributeDescription	*ad;
@@ -345,7 +326,6 @@ LDAPRDN_rewrite( LDAPRDN rdn, unsigned flags, void *ctx )
 		slap_syntax_transform_func *transf = NULL;
 		MatchingRule *mr = NULL;
 		struct berval		bv = BER_BVNULL;
-		int			do_sort = 0;
 
 		assert( ava != NULL );
 
@@ -445,10 +425,14 @@ LDAPRDN_rewrite( LDAPRDN rdn, unsigned flags, void *ctx )
 			ava->la_value = bv;
 			ava->la_flags |= LDAP_AVA_FREE_VALUE;
 		}
-
-		if( do_sort ) AVA_Sort( rdn, iAVA );
 	}
-	return LDAP_SUCCESS;
+	rc = LDAP_SUCCESS;
+
+	if ( do_sort ) {
+		rc = AVA_Sort( rdn, iAVA );
+	}
+
+	return rc;
 }
 
 /*
@@ -458,7 +442,7 @@ LDAPRDN_rewrite( LDAPRDN rdn, unsigned flags, void *ctx )
 static int
 LDAPDN_rewrite( LDAPDN dn, unsigned flags, void *ctx )
 {
-	int 		iRDN;
+	int 		iRDN, do_sort = 0;
 	int 		rc;
 
 	assert( dn != NULL );
@@ -477,7 +461,6 @@ LDAPDN_rewrite( LDAPDN dn, unsigned flags, void *ctx )
 			slap_syntax_transform_func *transf = NULL;
 			MatchingRule *mr = NULL;
 			struct berval		bv = BER_BVNULL;
-			int			do_sort = 0;
 
 			assert( ava != NULL );
 
@@ -578,10 +561,13 @@ LDAPDN_rewrite( LDAPDN dn, unsigned flags, void *ctx )
 				ava->la_flags |= LDAP_AVA_FREE_VALUE;
 			}
 
-			if( do_sort ) AVA_Sort( rdn, iAVA );
+		}
+		if( do_sort ) {
+			rc = AVA_Sort( rdn, iAVA );
+			if ( rc != LDAP_SUCCESS )
+				return rc;
 		}
 	}
-
 	return LDAP_SUCCESS;
 }
 
@@ -1410,7 +1396,8 @@ dnX509normalize( void *x509_name, struct berval *out )
 	int rc = ldap_X509dn2bv( x509_name, out, LDAPDN_rewrite, 0 );
 
 	Debug( LDAP_DEBUG_TRACE,
-		"dnX509Normalize: <%s>\n", out->bv_val, 0, 0 );
+		"dnX509Normalize: <%s> (%d)\n",
+		BER_BVISNULL( out ) ? "(null)" : out->bv_val, rc, 0 );
 
 	return rc;
 }
