@@ -1,8 +1,8 @@
 /* translucent.c - translucent proxy module */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/translucent.c,v 1.1.2.12 2006/12/13 16:40:52 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/translucent.c,v 1.1.2.14 2007/08/10 15:02:15 ghenry Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2004-2006 The OpenLDAP Foundation.
+ * Copyright 2004-2007 The OpenLDAP Foundation.
  * Portions Copyright 2005 Symas Corporation.
  * All rights reserved.
  *
@@ -686,7 +686,7 @@ static int translucent_db_init(BackendDB *be) {
 	overlay_stack *ov;
 	int rc;
 
-	Debug(LDAP_DEBUG_TRACE, "==> translucent_init\n", 0, 0, 0);
+	Debug(LDAP_DEBUG_TRACE, "==> translucent_db_init\n", 0, 0, 0);
 
 	ov = ch_calloc(1, sizeof(overlay_stack));
 	ov->config = ch_calloc(1, sizeof(translucent_configuration));
@@ -726,11 +726,11 @@ static int translucent_db_open(BackendDB *be) {
 
 	/* "should never happen" */
 	if(!ov->info) {
-		Debug(LDAP_DEBUG_ANY, "translucent_open() called with bad ov->info\n", 0, 0, 0);
+		Debug(LDAP_DEBUG_ANY, "translucent_db_open() called with bad ov->info\n", 0, 0, 0);
 		return(LDAP_OTHER);
 	}
 
-	Debug(LDAP_DEBUG_TRACE, "translucent_open\n", 0, 0, 0);
+	Debug(LDAP_DEBUG_TRACE, "translucent_db_open\n", 0, 0, 0);
 
 	be->be_private = ov->private;
 	rc = ov->info->bi_db_open ? ov->info->bi_db_open(be) : 0;
