@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/sets.c,v 1.24.2.3 2006/01/03 22:16:15 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/sets.c,v 1.24.2.7 2007/10/24 15:03:07 ando Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2000-2006 The OpenLDAP Foundation.
+ * Copyright 2000-2007 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -80,7 +80,7 @@ set_dup( SetCookie *cp, BerVarray set, unsigned flags )
 		for ( i = 0; !BER_BVISNULL( &set[ i ] ); i++ )
 			;
 		newset = cp->set_op->o_tmpcalloc( i + 1,
-				sizeof(struct berval), 
+				sizeof( struct berval ), 
 				cp->set_op->o_tmpmemctx );
 		if ( newset == NULL ) {
 			return NULL;
@@ -111,19 +111,19 @@ slap_set_join(
 	BerVarray	rset )
 {
 	BerVarray	set;
-	long		i, j, last;
+	long		i, j, last, rlast;
 	unsigned	op = ( op_flags & SLAP_SET_OPMASK );
 
 	set = NULL;
 	switch ( op ) {
 	case '|':	/* union */
-		if ( lset == NULL || BER_BVISNULL( lset ) ) {
+		if ( lset == NULL || BER_BVISNULL( &lset[ 0 ] ) ) {
 			if ( rset == NULL ) {
 				if ( lset == NULL ) {
 					set = cp->set_op->o_tmpcalloc( 1,
-							sizeof(struct berval),
+							sizeof( struct berval ),
 							cp->set_op->o_tmpmemctx );
-					BER_BVZERO( set );
+					BER_BVZERO( &set[ 0 ] );
 					return set;
 				}
 				return set_dup( cp, lset, SLAP_SET_LREF2REF( op_flags ) );
@@ -131,13 +131,15 @@ slap_set_join(
 			slap_set_dispose( cp, lset, SLAP_SET_LREF2REF( op_flags ) );
 			return set_dup( cp, rset, SLAP_SET_RREF2REF( op_flags ) );
 		}
-		if ( rset == NULL || BER_BVISNULL( rset ) ) {
+		if ( rset == NULL || BER_BVISNULL( &rset[ 0 ] ) ) {
 			slap_set_dispose( cp, rset, SLAP_SET_RREF2REF( op_flags ) );
 			return set_dup( cp, lset, SLAP_SET_LREF2REF( op_flags ) );
 		}
 
-		i = slap_set_size( lset ) + slap_set_size( rset ) + 1;
-		set = cp->set_op->o_tmpcalloc( i, sizeof(struct berval), cp->set_op->o_tmpmemctx );
+		/* worst scenario: no duplicates */
+		rlast = slap_set_size( rset );
+		i = slap_set_size( lset ) + rlast + 1;
+		set = cp->set_op->o_tmpcalloc( i, sizeof( struct berval ), cp->set_op->o_tmpmemctx );
 		if ( set != NULL ) {
 			/* set_chase() depends on this routine to
 			 * keep the first elements of the result
@@ -153,6 +155,9 @@ slap_set_join(
 				}
 			}
 
+			/* pointers to values have been used in set - don't free twice */
+			op_flags |= SLAP_SET_LREFVAL;
+
 			last = i;
 
 			for ( i = 0; !BER_BVISNULL( &rset[ i ] ); i++ ) {
@@ -163,11 +168,12 @@ slap_set_join(
 					{
 						if ( !( op_flags & SLAP_SET_RREFVAL ) ) {
 							cp->set_op->o_tmpfree( rset[ i ].bv_val, cp->set_op->o_tmpmemctx );
-							BER_BVZERO( &rset[ i ] );
+							rset[ i ] = rset[ --rlast ];
+							BER_BVZERO( &rset[ rlast ] );
 						}
 						exists = 1;
-						break;		
-					}	
+						break;
+					}
 				}
 
 				if ( !exists ) {
@@ -180,37 +186,59 @@ slap_set_join(
 					last++;
 				}
 			}
+
+			/* pointers to values have been used in set - don't free twice */
+			op_flags |= SLAP_SET_RREFVAL;
+
 			BER_BVZERO( &set[ last ] );
 		}
 		break;
 
 	case '&':	/* intersection */
-		if ( lset == NULL || BER_BVISNULL( lset )
-				|| rset == NULL || BER_BVISNULL( rset ) )
+		if ( lset == NULL || BER_BVISNULL( &lset[ 0 ] )
+			|| rset == NULL || BER_BVISNULL( &rset[ 0 ] ) )
 		{
-			set = cp->set_op->o_tmpcalloc( 1, sizeof(struct berval),
+			set = cp->set_op->o_tmpcalloc( 1, sizeof( struct berval ),
 					cp->set_op->o_tmpmemctx );
-			BER_BVZERO( set );
+			BER_BVZERO( &set[ 0 ] );
+			break;
 
 		} else {
-			set = set_dup( cp, lset, SLAP_SET_LREF2REF( op_flags ) );
+			long llen, rlen;
+			BerVarray sset;
+
+			llen = slap_set_size( lset );
+			rlen = slap_set_size( rset );
+
+			/* dup the shortest */
+			if ( llen < rlen ) {
+				last = llen;
+				set = set_dup( cp, lset, SLAP_SET_LREF2REF( op_flags ) );
+				lset = NULL;
+				sset = rset;
+
+			} else {
+				last = rlen;
+				set = set_dup( cp, rset, SLAP_SET_RREF2REF( op_flags ) );
+				rset = NULL;
+				sset = lset;
+			}
+
 			if ( set == NULL ) {
 				break;
 			}
-			lset = NULL;
-			last = slap_set_size( set ) - 1;
+
 			for ( i = 0; !BER_BVISNULL( &set[ i ] ); i++ ) {
-				for ( j = 0; !BER_BVISNULL( &rset[ j ] ); j++ ) {
-					if ( bvmatch( &set[ i ], &rset[ j ] ) ) {
+				for ( j = 0; !BER_BVISNULL( &sset[ j ] ); j++ ) {
+					if ( bvmatch( &set[ i ], &sset[ j ] ) ) {
 						break;
 					}
 				}
 
-				if ( BER_BVISNULL( &rset[ j ] ) ) {
+				if ( BER_BVISNULL( &sset[ j ] ) ) {
 					cp->set_op->o_tmpfree( set[ i ].bv_val, cp->set_op->o_tmpmemctx );
-					set[ i ] = set[ last ];
+					set[ i ] = set[ --last ];
 					BER_BVZERO( &set[ last ] );
-					last--;
 					i--;
 				}
 			}
@@ -221,7 +249,30 @@ slap_set_join(
 		i = slap_set_size( rset );
 		j = slap_set_size( lset );
 
-		set = cp->set_op->o_tmpcalloc( i * j + 1, sizeof(struct berval),
+		/* handle empty set cases */
+		if ( i == 0 ) {
+			if ( j == 0 ) {
+				set = cp->set_op->o_tmpcalloc( i * j + 1, sizeof( struct berval ),
+						cp->set_op->o_tmpmemctx );
+				if ( set == NULL ) {
+					break;
+				}
+				BER_BVZERO( &set[ 0 ] );
+				break;
+
+			} else {
+				set = set_dup( cp, lset, SLAP_SET_LREF2REF( op_flags ) );
+				lset = NULL;
+				break;
+			}
+
+		} else if ( j == 0 ) {
+			set = set_dup( cp, rset, SLAP_SET_RREF2REF( op_flags ) );
+			rset = NULL;
+			break;
+		}
+
+		set = cp->set_op->o_tmpcalloc( i * j + 1, sizeof( struct berval ),
 				cp->set_op->o_tmpmemctx );
 		if ( set == NULL ) {
 			break;
@@ -232,17 +283,36 @@ slap_set_join(
 				struct berval	bv;
 				long		k;
 
-				bv.bv_len = lset[ i ].bv_len + rset[ j ].bv_len;
-				bv.bv_val = cp->set_op->o_tmpalloc( bv.bv_len + 1,
-						cp->set_op->o_tmpmemctx );
-				if ( bv.bv_val == NULL ) {
-					slap_set_dispose( cp, set, 0 );
-					set = NULL;
-					goto done;
+				/* don't concatenate with the empty string */
+				if ( BER_BVISEMPTY( &lset[ i ] ) ) {
+					ber_dupbv_x( &bv, &rset[ j ], cp->set_op->o_tmpmemctx );
+					if ( bv.bv_val == NULL ) {
+						ber_bvarray_free_x( set, cp->set_op->o_tmpmemctx );
+						set = NULL;
+						goto done;
+					}
+
+				} else if ( BER_BVISEMPTY( &rset[ j ] ) ) {
+					ber_dupbv_x( &bv, &lset[ i ], cp->set_op->o_tmpmemctx );
+					if ( bv.bv_val == NULL ) {
+						ber_bvarray_free_x( set, cp->set_op->o_tmpmemctx );
+						set = NULL;
+						goto done;
+					}
+
+				} else {
+					bv.bv_len = lset[ i ].bv_len + rset[ j ].bv_len;
+					bv.bv_val = cp->set_op->o_tmpalloc( bv.bv_len + 1,
+							cp->set_op->o_tmpmemctx );
+					if ( bv.bv_val == NULL ) {
+						ber_bvarray_free_x( set, cp->set_op->o_tmpmemctx );
+						set = NULL;
+						goto done;
+					}
+					AC_MEMCPY( bv.bv_val, lset[ i ].bv_val, lset[ i ].bv_len );
+					AC_MEMCPY( &bv.bv_val[ lset[ i ].bv_len ], rset[ j ].bv_val, rset[ j ].bv_len );
+					bv.bv_val[ bv.bv_len ] = '\0';
 				}
-				AC_MEMCPY( bv.bv_val, lset[ i ].bv_val, lset[ i ].bv_len );
-				AC_MEMCPY( &bv.bv_val[ lset[ i ].bv_len ], rset[ j ].bv_val, rset[ j ].bv_len );
-				bv.bv_val[ bv.bv_len ] = '\0';
 
 				for ( k = 0; k < last; k++ ) {
 					if ( bvmatch( &set[ k ], &bv ) ) {
@@ -264,13 +334,8 @@ slap_set_join(
 	}
 
 done:;
-	if ( !( op_flags & SLAP_SET_LREFARR ) && lset != NULL ) {
-		cp->set_op->o_tmpfree( lset, cp->set_op->o_tmpmemctx );
-	}
-
-	if ( !( op_flags & SLAP_SET_RREFARR ) && rset != NULL ) {
-		cp->set_op->o_tmpfree( rset, cp->set_op->o_tmpmemctx );
-	}
+	if ( lset ) slap_set_dispose( cp, lset, SLAP_SET_LREF2REF( op_flags ) );
+	if ( rset ) slap_set_dispose( cp, rset, SLAP_SET_RREF2REF( op_flags ) );
 
 	return set;
 }
@@ -283,9 +348,11 @@ set_chase( SLAP_SET_GATHER gatherer,
 	int		i;
 
 	if ( set == NULL ) {
-		set = cp->set_op->o_tmpcalloc( 1, sizeof(struct berval),
+		set = cp->set_op->o_tmpcalloc( 1, sizeof( struct berval ),
 				cp->set_op->o_tmpmemctx );
-		BER_BVZERO( set );
+		if ( set != NULL ) {
+			BER_BVZERO( &set[ 0 ] );
+		}
 		return set;
 	}
 
@@ -293,22 +360,22 @@ set_chase( SLAP_SET_GATHER gatherer,
 		return set;
 	}
 
-	nset = cp->set_op->o_tmpcalloc( 1, sizeof(struct berval), cp->set_op->o_tmpmemctx );
+	nset = cp->set_op->o_tmpcalloc( 1, sizeof( struct berval ), cp->set_op->o_tmpmemctx );
 	if ( nset == NULL ) {
-		slap_set_dispose( cp, set, 0 );
+		ber_bvarray_free_x( set, cp->set_op->o_tmpmemctx );
 		return NULL;
 	}
 	for ( i = 0; !BER_BVISNULL( &set[ i ] ); i++ ) {
-		vals = (gatherer)( cp, &set[ i ], desc );
+		vals = gatherer( cp, &set[ i ], desc );
 		if ( vals != NULL ) {
 			nset = slap_set_join( cp, nset, '|', vals );
 		}
 	}
-	slap_set_dispose( cp, set, 0 );
+	ber_bvarray_free_x( set, cp->set_op->o_tmpmemctx );
 
 	if ( closure ) {
 		for ( i = 0; !BER_BVISNULL( &nset[ i ] ); i++ ) {
-			vals = (gatherer)( cp, &nset[ i ], desc );
+			vals = gatherer( cp, &nset[ i ], desc );
 			if ( vals != NULL ) {
 				nset = slap_set_join( cp, nset, '|', vals );
 				if ( nset == NULL ) {
@@ -329,13 +396,13 @@ slap_set_filter( SLAP_SET_GATHER gatherer,
 #define STACK_SIZE	64
 #define IS_SET(x)	( (unsigned long)(x) >= 256 )
 #define IS_OP(x)	( (unsigned long)(x) < 256 )
-#define SF_ERROR(x)	do { rc = -1; goto _error; } while (0)
-#define SF_TOP()	( (BerVarray)( (stp < 0) ? 0 : stack[ stp ] ) )
-#define SF_POP()	( (BerVarray)( (stp < 0) ? 0 : stack[ stp-- ] ) )
+#define SF_ERROR(x)	do { rc = -1; goto _error; } while ( 0 )
+#define SF_TOP()	( (BerVarray)( ( stp < 0 ) ? 0 : stack[ stp ] ) )
+#define SF_POP()	( (BerVarray)( ( stp < 0 ) ? 0 : stack[ stp-- ] ) )
 #define SF_PUSH(x)	do { \
-		if (stp >= (STACK_SIZE - 1)) SF_ERROR(overflow); \
+		if ( stp >= ( STACK_SIZE - 1 ) ) SF_ERROR( overflow ); \
 		stack[ ++stp ] = (BerVarray)(long)(x); \
-	} while (0)
+	} while ( 0 )
 
 	BerVarray	set, lset;
 	BerVarray	stack[ STACK_SIZE ] = { 0 };
@@ -380,7 +447,7 @@ slap_set_filter( SLAP_SET_GATHER gatherer,
 				SF_POP();
 				set = slap_set_join( cp, lset, op, set );
 				if ( set == NULL ) {
-					SF_ERROR(memory);
+					SF_ERROR( memory );
 				}
 				SF_PUSH( set );
 				set = NULL;
@@ -421,18 +488,18 @@ slap_set_filter( SLAP_SET_GATHER gatherer,
 			if ( ( SF_TOP() == (void *)'/' ) || IS_SET( SF_TOP() ) ) {
 				SF_ERROR( syntax );
 			}
-			for ( len = 0; ( c = *filter++ ) && (c != /* [ */ ']'); len++ )
+			for ( len = 0; ( c = *filter++ ) && ( c != /* [ */ ']' ); len++ )
 				;
 			if ( c == 0 ) {
-				SF_ERROR(syntax);
+				SF_ERROR( syntax );
 			}
 			
-			set = cp->set_op->o_tmpcalloc( 2, sizeof(struct berval),
+			set = cp->set_op->o_tmpcalloc( 2, sizeof( struct berval ),
 					cp->set_op->o_tmpmemctx );
 			if ( set == NULL ) {
-				SF_ERROR(memory);
+				SF_ERROR( memory );
 			}
-			set->bv_val = cp->set_op->o_tmpcalloc( len + 1, sizeof(char),
+			set->bv_val = cp->set_op->o_tmpcalloc( len + 1, sizeof( char ),
 					cp->set_op->o_tmpmemctx );
 			if ( BER_BVISNULL( set ) ) {
 				SF_ERROR( memory );
@@ -478,7 +545,7 @@ slap_set_filter( SLAP_SET_GATHER gatherer,
 				if ( ( SF_TOP() == (void *)'/' ) || IS_SET( SF_TOP() ) ) {
 					SF_ERROR( syntax );
 				}
-				set = cp->set_op->o_tmpcalloc( 2, sizeof(struct berval),
+				set = cp->set_op->o_tmpcalloc( 2, sizeof( struct berval ),
 						cp->set_op->o_tmpmemctx );
 				if ( set == NULL ) {
 					SF_ERROR( memory );
@@ -495,7 +562,7 @@ slap_set_filter( SLAP_SET_GATHER gatherer,
 				if ( ( SF_TOP() == (void *)'/' ) || IS_SET( SF_TOP() ) ) {
 					SF_ERROR( syntax );
 				}
-				set = cp->set_op->o_tmpcalloc( 2, sizeof(struct berval),
+				set = cp->set_op->o_tmpcalloc( 2, sizeof( struct berval ),
 						cp->set_op->o_tmpmemctx );
 				if ( set == NULL ) {
 					SF_ERROR( memory );
@@ -569,11 +636,11 @@ slap_set_filter( SLAP_SET_GATHER gatherer,
 
 _error:
 	if ( IS_SET( set ) ) {
-		slap_set_dispose( cp, set, 0 );
+		ber_bvarray_free_x( set, cp->set_op->o_tmpmemctx );
 	}
 	while ( ( set = SF_POP() ) ) {
 		if ( IS_SET( set ) ) {
-			slap_set_dispose( cp, set, 0 );
+			ber_bvarray_free_x( set, cp->set_op->o_tmpmemctx );
 		}
 	}
 	return rc;
